@@ -24,15 +24,18 @@ failing examples it finds.
 
 
 from hypothesis.searchstrategy.strategies import strategy
-from hypothesis.settings import Settings, Verbosity
+from hypothesis._settings import settings, Verbosity, Settings
 from hypothesis.version import __version_info__, __version__
 from hypothesis.control import assume, note
-from hypothesis.core import given, find, example
+from hypothesis.core import given, find, example, seed
+
 
 __all__ = [
+    'settings',
     'Settings',
     'Verbosity',
     'assume',
+    'seed',
     'given',
     'strategy',
     'find',
