@@ -36,7 +36,8 @@ from hypothesis.errors import Flaky, Timeout, NoSuchExample, \
     UnsatisfiedAssumption, DefinitelyNoSuchExample, \
     HypothesisDeprecationWarning
 from hypothesis.control import BuildContext
-from hypothesis.settings import Settings, Verbosity, note_deprecation
+from hypothesis._settings import settings as Settings
+from hypothesis._settings import Verbosity, note_deprecation
 from hypothesis.executors import executor, default_executor
 from hypothesis.reporting import report, debug_report, verbose_report, \
     current_verbosity
@@ -201,7 +202,10 @@ def simplify_template_such_that(
     """
     yield t
 
-    if settings.max_shrinks <= 0 or not f(t):
+    try:
+        if settings.max_shrinks <= 0 or not f(t):
+            return
+    except UnsatisfiedAssumption:
         return
 
     successful_shrinks = 0
@@ -377,6 +381,22 @@ def reify_and_execute(
     return run
 
 
+def seed(seed):
+    """
+    seed: Start the test execution from a specific seed. May be any hashable
+          object. No exact meaning for seed is provided other than that
+          for a fixed seed value Hypothesis will try the same actions (insofar
+          as it can given external sources of non-determinism. e.g. timing and
+          hash randomization).
+          Overrides the derandomize setting if it is present.
+    """
+
+    def accept(test):
+        test._hypothesis_internal_use_seed = seed
+        return test
+    return accept
+
+
 def given(*generator_arguments, **generator_kwargs):
     """A decorator for turning a test function that accepts arguments into a
     randomized test.
@@ -385,19 +405,12 @@ def given(*generator_arguments, **generator_kwargs):
     for details of its behaviour.
 
     """
-
-    # Keyword only arguments but actually supported in the full range of
-    # pythons Hypothesis handles. pop so we don't later pick these up as
-    # if they were keyword specifiers for data to pass to the test.
-    provided_random = generator_kwargs.pop(u'random', None)
-    settings = generator_kwargs.pop(u'settings', None) or Settings.default
-    if generator_arguments and generator_kwargs:
-        note_deprecation(
-            u'Mixing positional and keyword arguments in a call to given is '
-            u'deprecated. Use one or the other.', settings
-        )
-
     def run_test_with_generator(test):
+        # Keyword only arguments but actually supported in the full range of
+        # pythons Hypothesis handles. pop so we don't later pick these up as
+        # if they were keyword specifiers for data to pass to the test.
+        _provided_random = generator_kwargs.pop(u'random', None)
+        _settings = generator_kwargs.pop(u'settings', None)
         original_argspec = getargspec(test)
 
         def invalid(message):
@@ -405,10 +418,6 @@ def given(*generator_arguments, **generator_kwargs):
                 raise InvalidArgument(message)
             return wrapped_test
 
-        if (provided_random is not None) and settings.derandomize:
-            return invalid(
-                u'Cannot both be derandomized and provide an explicit random')
-
         if not (generator_arguments or generator_kwargs):
             return invalid(
                 u'given must be called with at least one argument')
@@ -476,10 +485,34 @@ def given(*generator_arguments, **generator_kwargs):
             test.__name__, argspec
         )
         def wrapped_test(*arguments, **kwargs):
-            if settings.derandomize:
+            if _provided_random is not None:
+                note_deprecation(
+                    'Passing random as an argument to @given is deprecated. '
+                    'use the @seed() API to pass an explicit seed instead.',
+                    _settings or getattr(
+                        test, '_hypothesis_internal_use_settings', None))
+            if _settings is not None:
+                note_deprecation(
+                    'Passing settings as an argument to @given is deprecated. '
+                    'use Settings as a decorator directly instead.',
+                    _settings or getattr(
+                        test, '_hypothesis_internal_use_settings', None))
+            if generator_arguments and generator_kwargs:
+                note_deprecation(
+                    'Mixing positional and keyword arguments in a call to '
+                    'given is deprecated. Use one or the other.', getattr(
+                        test, '_hypothesis_internal_use_settings', _settings,
+                    )
+                )
+
+            settings = wrapped_test._hypothesis_internal_use_settings
+            if wrapped_test._hypothesis_internal_use_seed is not None:
+                random = Random(
+                    wrapped_test._hypothesis_internal_use_seed)
+            elif settings.derandomize:
                 random = Random(function_digest(test))
             else:
-                random = provided_random or new_random()
+                random = new_random()
 
             import hypothesis.strategies as sd
             from hypothesis.internal.strategymethod import strategy
@@ -598,15 +631,16 @@ def given(*generator_arguments, **generator_kwargs):
             if perform_health_check:
                 initial_state = getglobalrandomstate()
                 health_check_random = Random(random.getrandbits(128))
-                with Settings(settings, verbosity=Verbosity.quiet):
-                    count = 0
-                    bad_draws = 0
-                    filtered_draws = 0
-                    while (
-                        count < 10 and time.time() < start + 1 and
-                        filtered_draws < 50 and bad_draws < 50
-                    ):
-                        try:
+                count = 0
+                bad_draws = 0
+                filtered_draws = 0
+                errors = 0
+                while (
+                    count < 10 and time.time() < start + 1 and
+                    filtered_draws < 50 and bad_draws < 50
+                ):
+                    try:
+                        with Settings(settings, verbosity=Verbosity.quiet):
                             test_runner(reify_and_execute(
                                 search_strategy,
                                 search_strategy.draw_template(
@@ -616,34 +650,36 @@ def given(*generator_arguments, **generator_kwargs):
                                     )),
                                 lambda *args, **kwargs: None,
                             ))
-                            count += 1
-                        except BadTemplateDraw:
-                            bad_draws += 1
-                        except UnsatisfiedAssumption:
-                            filtered_draws += 1
-                        except Exception:
-                            traceback.print_exc()
-                            if test_runner is default_executor:
-                                fail_health_check(
-                                    'An exception occurred during data '
-                                    'generation in initial health check. '
-                                    'This indicates a bug in the strategy. '
-                                    'This could either be a Hypothesis bug or '
-                                    "an error in a function you've passed to "
-                                    'it to construct your data.'
-                                )
-                            else:
-                                fail_health_check(
-                                    'An exception occurred during data '
-                                    'generation in initial health check. '
-                                    'This indicates a bug in the strategy. '
-                                    'This could either be a Hypothesis bug or '
-                                    'an error in a function you\'ve passed to '
-                                    'it to construct your data. Additionally, '
-                                    'you have a custom executor, which means '
-                                    'that this could be your executor failing '
-                                    'to handle a function which returns None. '
-                                )
+                        count += 1
+                    except BadTemplateDraw:
+                        bad_draws += 1
+                    except UnsatisfiedAssumption:
+                        filtered_draws += 1
+                    except Exception:
+                        if errors == 0:
+                            report(traceback.format_exc())
+                        errors += 1
+                        if test_runner is default_executor:
+                            fail_health_check(
+                                'An exception occurred during data '
+                                'generation in initial health check. '
+                                'This indicates a bug in the strategy. '
+                                'This could either be a Hypothesis bug or '
+                                "an error in a function you've passed to "
+                                'it to construct your data.'
+                            )
+                        else:
+                            fail_health_check(
+                                'An exception occurred during data '
+                                'generation in initial health check. '
+                                'This indicates a bug in the strategy. '
+                                'This could either be a Hypothesis bug or '
+                                'an error in a function you\'ve passed to '
+                                'it to construct your data. Additionally, '
+                                'you have a custom executor, which means '
+                                'that this could be your executor failing '
+                                'to handle a function which returns None. '
+                            )
                 if filtered_draws >= 50:
                     fail_health_check((
                         'It looks like your strategy is filtering out a lot '
@@ -729,37 +765,64 @@ def given(*generator_arguments, **generator_kwargs):
             is_template_example.__name__ = test.__name__
             is_template_example.__qualname__ = qualname(test)
 
-            falsifying_template = None
-            try:
-                falsifying_template = best_satisfying_template(
-                    search_strategy, random, is_template_example,
-                    settings, storage, start_time=start,
-                )
-            except NoSuchExample:
-                return
-
-            assert last_exception[0] is not None
-
             with settings:
-                test_runner(reify_and_execute(
-                    search_strategy, falsifying_template, test,
-                    print_example=True, is_final=True
-                ))
+                falsifying_template = None
+                try:
+                    falsifying_template = best_satisfying_template(
+                        search_strategy, random, is_template_example,
+                        settings, storage, start_time=start,
+                    )
+                except NoSuchExample:
+                    return
+
+                assert last_exception[0] is not None
+
+                try:
+                    test_runner(reify_and_execute(
+                        search_strategy, falsifying_template, test,
+                        print_example=True, is_final=True
+                    ))
+                except UnsatisfiedAssumption:
+                    report(traceback.format_exc())
+                    raise Flaky(
+                        'Unreliable assumption: An example which satisfied '
+                        'assumptions on the first run now fails it.'
+                    )
 
                 report(
                     u'Failed to reproduce exception. Expected: \n' +
                     last_exception[0],
                 )
 
-                test_runner(reify_and_execute(
-                    search_strategy, falsifying_template,
-                    test_is_flaky(test, repr_for_last_exception[0]),
-                    print_example=True, is_final=True
-                ))
+                try:
+                    test_runner(reify_and_execute(
+                        search_strategy, falsifying_template,
+                        test_is_flaky(test, repr_for_last_exception[0]),
+                        print_example=True, is_final=True
+                    ))
+                except UnsatisfiedAssumption:
+                    raise Flaky(
+                        'Unreliable test data: Failed to reproduce a failure '
+                        'and then when it came to recreating the example in '
+                        'order to print the test data with a flaky result '
+                        'the example was filtered out (by e.g. a '
+                        'call to filter in your strategy) when we didn\'t '
+                        'expect it to be.'
+                    )
         for attr in dir(test):
             if attr[0] != '_' and not hasattr(wrapped_test, attr):
                 setattr(wrapped_test, attr, getattr(test, attr))
         wrapped_test.is_hypothesis_test = True
+        if _provided_random is not None:
+            wrapped_test._hypothesis_internal_use_seed = \
+                _provided_random.getrandbits(128)
+        else:
+            wrapped_test._hypothesis_internal_use_seed = getattr(
+                test, '_hypothesis_internal_use_seed', None
+            )
+        wrapped_test._hypothesis_internal_use_settings = _settings or getattr(
+            test, '_hypothesis_internal_use_settings', None
+        ) or Settings.default
         return wrapped_test
     return run_test_with_generator
 
